////////////////////////////////////////////////////////////////////////////////
///
///		Title		: NPhoto Singleton
///
///		Copyright (c) 2006, NHN Co., Ltd. ALL RIGHTS RESERVED
///
///		Team		: Desktop Service Development
///
////////////////////////////////////////////////////////////////////////////////
#pragma once

#include "NPSyncObject.h"

/**
* @group 	PCM Base Lib	
* @brief 	Singleton for Threadsafe
* @date		08/21/2006
* @author	telltale
* @remark	Example\n
NPXXX.h\n
class CNPXXX\n
{\n
	DECLARE_SINGLETON_CODE(CNPXXX)\n
}\n
NPXXX.cpp\n
IMPLEMENT_SINGLETON_CODE(CNPXXX)\n
\n
Usage\n
CNPXXX* pNPXXX = CNPXXX::get_instance();\n
\n
//destroy\n
CNPXXX::release_instance();\n
*/
template < class T > 
class CNPSingleton
{
public:
	/**
	* @brief 	{kr} Single Instance ´.	{/kr}
	*/
	static	T* get_instance()
	{
		CNPLock guard(m_cs);	// For Multi Threading.

		if( m_pSelfInstance == 0 )
		{
			m_pSelfInstance = new T();
		}
		return m_pSelfInstance;

	}

	/**
	* @brief 	{kr} Single Instance Ѵ.	{/kr}
	*/
	static	void	release_instance()
	{
		if( m_pSelfInstance != 0 )
			delete m_pSelfInstance;

		m_pSelfInstance = 0;
	}
private:
	static	T*		m_pSelfInstance;
	static	CNPCriticalSection m_cs;
};

/**
@brief : {kr} ThreadSafe   Singleton
Ŭ ǿ DECLARE_SINGLETON_CODE(<class_name>) ְ
cpp IMPLEMENT_SINGLETON_CODE(<class_name>)ش. {/kr}
*/
template < class T > T*		CNPSingleton< T >::m_pSelfInstance = 0;
template<typename T> CNPCriticalSection CNPSingleton<T>::m_cs;

#define DECLARE_SINGLETON_CODE(Type)					\
	protected:											\
	static Type * m_pSelfInstance;					\
	\
	public:												\
	static Type * get_instance() {				\
	if(m_pSelfInstance == NULL ) {          \
	m_pSelfInstance =  new Type();		\
	}										\
	\
	return m_pSelfInstance;					\
	}											\
	static	void	release_instance()			\
{											\
	if( m_pSelfInstance != 0 )				\
	delete m_pSelfInstance;				\
	\
	m_pSelfInstance = 0;					\
}											\



#define IMPLEMENT_SINGLETON_CODE(Type)					\
	Type* Type::m_pSelfInstance = 0;	